#include <iostream>
using namespace std;
// -----------------------------------------------------------------------------

#include "hchain.h"
// -----------------------------------------------------------------------------

hash_chain::hash_chain(int __size, int (*__hashfunc)(int))
{
    _valtab = NULL;
    _sizetab = NULL;
    _size = 0;
    _hash = NULL;

    if(__size <= 0)
        return;
    if(__hashfunc == NULL)
        return;

    _valtab = new int*[__size];
    _sizetab = new int[__size];
    _size = __size;
    _hash = __hashfunc;

    for(int i=0;i<_size;++i)
    {
        _valtab[i] = NULL;
        _sizetab[i] = 0;
    }
}
// -----------------------------------------------------------------------------

hash_chain::~hash_chain(void)
{
    for(int i=0;i<_size;++i)
    {
        if(_sizetab[i] > 0)
            delete _valtab[i];
    }
    delete [] _valtab;
    delete [] _sizetab;

    // aby zabezpieczyc przed jawnym wywolaniem destruktora
    _valtab = NULL;
    _sizetab = NULL;
    _size = 0;
}
// -----------------------------------------------------------------------------

bool hash_chain::changeListSize(int __listindex, bool __increase)
{
    if((__listindex < 0) || (__listindex >= _size))
        return false;

    int d = __increase?1:-1;
    int cursize = _sizetab[__listindex];
    int newsize = cursize+d;
    int* tmp = new int[newsize];
    int copycount = __increase?cursize:newsize;

    for(int i=0;i<copycount;++i)
        tmp[i] = _valtab[__listindex][i];
    if(cursize > 0)
        delete [] _valtab[__listindex];
    _valtab[__listindex] = tmp;
    _sizetab[__listindex] = newsize;

    return true;
}
// -----------------------------------------------------------------------------

int hash_chain::findOnTheList(int __listindex, int __val)
{
    if((__listindex < 0) || (__listindex >= _size))
        return -1;

    for(int i=0;i<_sizetab[__listindex];++i)
        if(_valtab[__listindex][i] == __val)
            return i;

    return -1;
}
// -----------------------------------------------------------------------------

bool hash_chain::insert(int __val)
{
    int listindex = _hash(__val);
    if((listindex < 0) || (listindex >= _size))
        return false;

    if(changeListSize(listindex, true))
        _valtab[listindex][_sizetab[listindex]-1] = __val;
    else
        return false;

    return true;
}
// -----------------------------------------------------------------------------

bool hash_chain::remove(int __val)
{
    int listindex = _hash(__val);
    if((listindex < 0) || (listindex >= _size))
        return false;

    int valindex = findOnTheList(listindex, __val);
    if(valindex == -1)
        return true;
    for(int i=valindex;i<(_sizetab[listindex]-1);++i)
        _valtab[listindex][i] = _valtab[listindex][i+1];

    if(!changeListSize(listindex, false))
        return false;
    return true;
}
// -----------------------------------------------------------------------------

bool hash_chain::exists(int __val)
{
    int listindex = _hash(__val);
    if((listindex < 0) || (listindex >= _size))
        return false;

    return findOnTheList(listindex, __val) > -1;
}
// ---------------------------------------------------------------------------

void hash_chain::printValues(void)
{
    for(int l=0;l<_size;++l)
    {
        cout << "Lista[" << l << "]: " << _sizetab[l] << " el: ";
        for(int i=0;i<_sizetab[l];++i)
        {
            cout << _valtab[l][i] << ", ";
        }
        cout << endl;
    }
}
// ---------------------------------------------------------------------------

void hash_chain::printInfo(void)
{
    int esum = 0;
    int emax = -1;
    int emin = -1;
    for(int i=0;i<_size;++i)
    {
        if(emax < _sizetab[i])
            emax = _sizetab[i];
        if((emin > _sizetab[i]) || (emin == -1))
            emin = _sizetab[i];
        esum += _sizetab[i];
    }
    double avecount = (double)esum/(double)_size;

    double stddev = .0;
    for(int i=0;i<_size;++i)
    {
        double dev = avecount-_sizetab[i];
        dev = dev<0.?-dev:dev;
        stddev += dev;
    }
    stddev /= (double)_size;

    cout << "Srednia ilosc elementow: " << avecount << endl;
    cout << "Maksymalna elementow: " << emax << endl;
    cout << "Minimalna elementow: " << emin << endl;
    cout << "Srednie odchylenie od sredniej: " << stddev << endl;
}
// ---------------------------------------------------------------------------
